/*****************************************************************************
 * HBgEsti.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HBgEsti
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 09:52:18 $
 *
 */

#ifndef H_BG_ESTI_H
#define H_BG_ESTI_H

class LIntExport HBgEsti: public HRootObject {
public:
  HBgEsti(void) {id = -1;}
  HBgEsti(const HImage &InitializeImage, 
          const HTuple &Syspar1 = 0.7, const HTuple &Syspar2 = 0.7, 
          const HTuple &GainMode = "fixed", 
          const HTuple &Gain1 = 0.002, const HTuple &Gain2 = 0.002, 
          const HTuple &AdaptMode = "on", 
          const HTuple &MinDiff = 7, 
          const HTuple &StatNum = 10, 
          const HTuple &ConfidenceC = 3.25, 
          const HTuple &TimeC = 15);
  HBgEsti(const HImage &InitializeImage, 
          double        Syspar1 = 0.7, double Syspar2 = 0.7, 
          const char   *GainMode = "fixed", 
          double        Gain1 = 0.002, double Gain2 = 0.002, 
          const char   *AdaptMode = "on", 
          double        MinDiff = 7, 
          long          StatNum = 10, 
          double        ConfidenceC = 3.25, 
          double        TimeC = 15);

  ~HBgEsti(void);
private:
  // Copying and Assignment not allowed
  HBgEsti(const HBgEsti &inst);
  HBgEsti &operator= (const HBgEsti &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HBgEsti";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Return the estimated background image.
  virtual HImage GiveBgEsti() const;
  // Change the estimated background image.
  virtual void UpdateBgEsti(const HImage &PresentImage, const HRegion &UpDateRegion) const;
  // Estimate the background and return the foreground region.
  virtual HRegion RunBgEsti(const HImage &PresentImage) const;
  // Return the parameters of the data set.
  virtual double GetBgEstiParams(HTuple *Syspar2, HTuple *GainMode, HTuple *Gain1, HTuple *Gain2, HTuple *AdaptMode, HTuple *MinDiff, HTuple *StatNum, HTuple *ConfidenceC, HTuple *TimeC) const;
  virtual double GetBgEstiParams(double *Syspar2, char *GainMode, double *Gain1, double *Gain2, char *AdaptMode, double *MinDiff, long *StatNum, double *ConfidenceC, double *TimeC) const;
  // Change the parameters of the data set.
  virtual void SetBgEstiParams(const HTuple &Syspar1, const HTuple &Syspar2, const HTuple &GainMode, const HTuple &Gain1, const HTuple &Gain2, const HTuple &AdaptMode, const HTuple &MinDiff, const HTuple &StatNum, const HTuple &ConfidenceC, const HTuple &TimeC) const;
  virtual void SetBgEstiParams(double Syspar1, double Syspar2, const char *GainMode, double Gain1, double Gain2, const char *AdaptMode, double MinDiff, long StatNum, double ConfidenceC, double TimeC) const;
  // Generate and initialize a data set for the background estimation.
  virtual void CreateBgEsti(const HImage &InitializeImage, const HTuple &Syspar1, const HTuple &Syspar2, const HTuple &GainMode, const HTuple &Gain1, const HTuple &Gain2, const HTuple &AdaptMode, const HTuple &MinDiff, const HTuple &StatNum, const HTuple &ConfidenceC, const HTuple &TimeC);
  virtual void CreateBgEsti(const HImage &InitializeImage, double Syspar1, double Syspar2, const char *GainMode, double Gain1, double Gain2, const char *AdaptMode, double MinDiff, long StatNum, double ConfidenceC, double TimeC);
};

#endif
