/*****************************************************************************
 * HGnuplot.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HGnuplot
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 09:56:04 $
 *
 */

#ifndef H_GNUPLOT_H
#define H_GNUPLOT_H

class LIntExport HGnuplot: public HRootObject {
public:
  HGnuplot(void) {id = -1;}
  HGnuplot(const HTuple &FileName);
  HGnuplot(const char *FileName);

  ~HGnuplot(void);
private:
  // Copying and Assignment not allowed
  HGnuplot(const HGnuplot &inst);
  HGnuplot &operator= (const HGnuplot &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HGnuplot";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Plot a function using gnuplot.
  virtual void GnuplotPlotFunct1d(const HTuple &Function) const;
  // Plot control values using gnuplot.
  virtual void GnuplotPlotCtrl(const HTuple &Values) const;
  // Visualize images using gnuplot.
  virtual void GnuplotPlotImage(const HImage &Image, const HTuple &SamplesX, const HTuple &SamplesY, const HTuple &ViewRotX, const HTuple &ViewRotZ, const HTuple &Hidden3D) const;
  virtual void GnuplotPlotImage(const HImage &Image, long SamplesX, long SamplesY, double ViewRotX, double ViewRotZ, const char *Hidden3D) const;
  // Open a gnuplot file for visualization of images and control values.
  virtual void GnuplotOpenFile(const HTuple &FileName);
  virtual void GnuplotOpenFile(const char *FileName);
  // Open a pipe to a gnuplot process for visualization of images and control
  // values.
  virtual void GnuplotOpenPipe(void);
};

#endif
