/*****************************************************************************
 * HImageArray.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Types for Halcon-images arrays
 *
 * (c) 1996-2000 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.12 $
 * $Date: 2000/07/25 08:17:53 $
 *
 */

#ifndef H_IMAGE_ARRAY_H
#define H_IMAGE_ARRAY_H


/****************************************************************************/
/*                             HImageArray                                  */
/****************************************************************************/
class LIntExport HImageArray: public HObjectArray {
public:
  HImageArray(void) : array(NULL) {num_image_arrays++;}
  HImageArray(Hkey *key, long num);
  HImageArray(const Hobject &obj);
  HImageArray(const HImage &image);
  HImageArray(const HImageArray &arr);
  ~HImageArray(void);
  HImageArray &operator = (const HImageArray &arr);
  
  // Name der Halcon-Klasse -> get_obj_class
  virtual const char *HClassName(void)    const;
  virtual const char *ClassName(void)     const;
  virtual       INT4  ClassDBID(void)     const {return IMAGE_ID;}
  virtual       Hkey  Key(long i)         const {return array[i].Key();}
  virtual    Hobject  ElemId(long i)      const {return array[i].Id();}
  virtual       void  CopyHImageArray(const HImageArray &in);
  virtual       void  Display(const HWindow &w) const;

  
  // Zugriff und Manipulation
  HImage       &operator [] (long index);
  HImage const &operator [] (long index) const;
  HImageArray  operator () (long min, long max);
  HImageArray  &Append(const HImage &image);
  HImageArray  &Append(const HImageArray &images);
  virtual void SetKeys(Hkey *key, long num);
  virtual void DeleteArray(void);

  // Klassenoperationen
  static long NumInst(void) { return num_image_arrays; }
private:
  static long num_image_arrays;
  HImage  *array;
  void    NewLength(long new_num, long old_num);
public:
  // Convert one-channel images into a multichannel image
  virtual HImage ChannelsToImage(void) const;
  virtual HImageArray Compose7(const HImageArray &Image2, const HImageArray &Image3, const HImageArray &Image4, const HImageArray &Image5, const HImageArray &Image6, const HImageArray &Image7) const;
  virtual HImageArray Compose6(const HImageArray &Image2, const HImageArray &Image3, const HImageArray &Image4, const HImageArray &Image5, const HImageArray &Image6) const;
  virtual HImageArray Compose5(const HImageArray &Image2, const HImageArray &Image3, const HImageArray &Image4, const HImageArray &Image5) const;
  virtual HImageArray Compose4(const HImageArray &Image2, const HImageArray &Image3, const HImageArray &Image4) const;
  virtual HImageArray Compose3(const HImageArray &Image2, const HImageArray &Image3) const;
  virtual HImageArray Compose2(const HImageArray &Image2) const;
  virtual HImageArray Decompose7(HImageArray *Image2, HImageArray *Image3, HImageArray *Image4, HImageArray *Image5, HImageArray *Image6, HImageArray *Image7) const;
  virtual HImageArray Decompose6(HImageArray *Image2, HImageArray *Image3, HImageArray *Image4, HImageArray *Image5, HImageArray *Image6) const;
  virtual HImageArray Decompose5(HImageArray *Image2, HImageArray *Image3, HImageArray *Image4, HImageArray *Image5) const;
  virtual HImageArray Decompose4(HImageArray *Image2, HImageArray *Image3, HImageArray *Image4) const;
  virtual HImageArray Decompose3(HImageArray *Image2, HImageArray *Image3) const;
  virtual HImageArray Decompose2(HImageArray *Image2) const;
  virtual HTuple CountChannels(void) const;
  virtual HImage TileImagesOffset(const HTuple &OffsetRow, const HTuple &OffsetCol, const HTuple &Row1, const HTuple &Col1, const HTuple &Row2, const HTuple &Col2, const HTuple &Width, const HTuple &Height) const;
  virtual HImage TileImagesOffset(long OffsetRow, long OffsetCol, long Row1, long Col1, long Row2, long Col2, long Width, long Height) const;
  // Tile multiple image objects into a large image.
  virtual HImage TileImages(const HTuple &NumColumns, const HTuple &TileOrder) const;
  virtual HImage TileImages(long NumColumns, const char *TileOrder) const;
  virtual HImageArray TileChannels(const HTuple &NumColumns, const HTuple &TileOrder) const;
  virtual HImageArray TileChannels(long NumColumns, const char *TileOrder) const;
  virtual HImageArray CropDomain(void) const;
  virtual HImageArray CropRectangle1(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual HImageArray CropRectangle1(long Row1, long Column1, long Row2, long Column2) const;
  virtual HImageArray CropPart(const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height) const;
  virtual HImageArray CropPart(long Row, long Column, long Width, long Height) const;
  virtual HImageArray ChangeFormat(const HTuple &Width, const HTuple &Height) const;
  virtual HImageArray ChangeFormat(long Width, long Height) const;
  virtual HImageArray ChangeDomain(const HRegion &NewDomain) const;
  virtual HImageArray Rectangle1Domain(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual HImageArray Rectangle1Domain(long Row1, long Column1, long Row2, long Column2) const;
  virtual HImageArray ReduceDomain(const HRegion &Region) const;
  virtual HImageArray FullDomain(void) const;
  virtual HRegionArray GetDomain(void) const;
  virtual void WriteImage(const HTuple &Format, const HTuple &FillColor, const HTuple &FileName) const;
  virtual void WriteImage(const char *Format, long FillColor, const char *FileName) const;
  // Combine multiple images into a mosaic image.
  virtual HImage GenProjectiveMosaic(const HTuple &StartImage, const HTuple &MappingSource, const HTuple &MappingDest, const HTuple &HomMatrices2D, const HTuple &StackingOrder, const HTuple &TransformRegion, HTuple *MosaicMatrices2D) const;
  virtual HImageArray ProjectiveTransImageSize(const HTuple &HomMat2D, const HTuple &Interpolation, const HTuple &Width, const HTuple &Height, const HTuple &TransformRegion) const;
  virtual HImageArray ProjectiveTransImageSize(double HomMat2D, const char *Interpolation, long Width, long Height, const char *TransformRegion) const;
  virtual HImageArray ProjectiveTransImage(const HTuple &HomMat2D, const HTuple &Interpolation, const HTuple &AdaptImageSize, const HTuple &TransformRegion) const;
  virtual HImageArray ProjectiveTransImage(double HomMat2D, const char *Interpolation, const char *AdaptImageSize, const char *TransformRegion) const;
  virtual HImageArray AffineTransImageSize(const HTuple &HomMat2D, const HTuple &Interpolation, const HTuple &Width, const HTuple &Height) const;
  virtual HImageArray AffineTransImageSize(double HomMat2D, const char *Interpolation, long Width, long Height) const;
  virtual HImageArray AffineTransImage(const HTuple &HomMat2D, const HTuple &Interpolation, const HTuple &AdaptImageSize) const;
  virtual HImageArray AffineTransImage(double HomMat2D, const char *Interpolation, const char *AdaptImageSize) const;
  virtual HImageArray ZoomImageFactor(const HTuple &ScaleWidth, const HTuple &ScaleHeight, const HTuple &Interpolation) const;
  virtual HImageArray ZoomImageFactor(double ScaleWidth, double ScaleHeight, const char *Interpolation) const;
  virtual HImageArray ZoomImageSize(const HTuple &Width, const HTuple &Height, const HTuple &Interpolation) const;
  virtual HImageArray ZoomImageSize(long Width, long Height, const char *Interpolation) const;
  virtual HImageArray MirrorImage(const HTuple &Mode) const;
  virtual HImageArray MirrorImage(const char *Mode) const;
  virtual HImageArray RotateImage(const HTuple &Phi, const HTuple &Interpolation) const;
  virtual HImageArray RotateImage(double Phi, const char *Interpolation) const;
  virtual HImageArray PolarTransImageInv(const HTuple &Row, const HTuple &Column, const HTuple &AngleStart, const HTuple &AngleEnd, const HTuple &RadiusStart, const HTuple &RadiusEnd, const HTuple &Width, const HTuple &Height, const HTuple &Interpolation) const;
  virtual HImageArray PolarTransImageInv(double Row, double Column, double AngleStart, double AngleEnd, double RadiusStart, double RadiusEnd, long Width, long Height, const char *Interpolation) const;
  virtual HImageArray PolarTransImageExt(const HTuple &Row, const HTuple &Column, const HTuple &AngleStart, const HTuple &AngleEnd, const HTuple &RadiusStart, const HTuple &RadiusEnd, const HTuple &Width, const HTuple &Height, const HTuple &Interpolation) const;
  virtual HImageArray PolarTransImageExt(double Row, double Column, double AngleStart, double AngleEnd, double RadiusStart, double RadiusEnd, long Width, long Height, const char *Interpolation) const;
  virtual HImageArray PolarTransImage(const HTuple &Row, const HTuple &Column, const HTuple &Width, const HTuple &Height) const;
  virtual HImageArray PolarTransImage(long Row, long Column, long Width, long Height) const;
  virtual HImageArray SqrtImage(void) const;
  virtual HImageArray SubImage(const HImageArray &ImageSubtrahend, const HTuple &Mult, const HTuple &Add) const;
  virtual HImageArray SubImage(const HImageArray &ImageSubtrahend, double Mult, double Add) const;
  virtual HImageArray ScaleImage(const HTuple &Mult, const HTuple &Add) const;
  virtual HImageArray ScaleImage(double Mult, double Add) const;
  virtual HImageArray DivImage(const HImageArray &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImageArray DivImage(const HImageArray &Image2, double Mult, double Add) const;
  virtual HImageArray MultImage(const HImageArray &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImageArray MultImage(const HImageArray &Image2, double Mult, double Add) const;
  virtual HImageArray AddImage(const HImageArray &Image2, const HTuple &Mult, const HTuple &Add) const;
  virtual HImageArray AddImage(const HImageArray &Image2, double Mult, double Add) const;
  virtual HImageArray AbsImage(void) const;
  virtual HImageArray MinImage(const HImageArray &Image2) const;
  virtual HImageArray MaxImage(const HImageArray &Image2) const;
  virtual HImageArray InvertImage(void) const;
  virtual HImageArray BitMask(const HTuple &BitMask) const;
  virtual HImageArray BitMask(long BitMask) const;
  virtual HImageArray BitSlice(const HTuple &Bit) const;
  virtual HImageArray BitSlice(long Bit) const;
  virtual HImageArray BitRshift(const HTuple &Shift) const;
  virtual HImageArray BitRshift(long Shift) const;
  virtual HImageArray BitLshift(const HTuple &Shift) const;
  virtual HImageArray BitLshift(long Shift) const;
  virtual HImageArray BitNot(void) const;
  virtual HImageArray BitXor(const HImageArray &Image2) const;
  virtual HImageArray BitOr(const HImageArray &Image2) const;
  virtual HImageArray BitAnd(const HImageArray &Image2) const;
  virtual HImageArray Rgb1ToGray(void) const;
  virtual HImageArray Rgb3ToGray(const HImageArray &ImageGreen, const HImageArray &ImageBlue) const;
  virtual HImageArray TransFromRgb(const HImageArray &ImageGreen, const HImageArray &ImageBlue, HImageArray *ImageResult2, HImageArray *ImageResult3, const HTuple &ColorSpace) const;
  virtual HImageArray TransFromRgb(const HImageArray &ImageGreen, const HImageArray &ImageBlue, HImageArray *ImageResult2, HImageArray *ImageResult3, const char *ColorSpace) const;
  virtual HImageArray TransToRgb(const HImageArray &ImageInput2, const HImageArray &ImageInput3, HImageArray *ImageGreen, HImageArray *ImageBlue, const HTuple &ColorSpace) const;
  virtual HImageArray TransToRgb(const HImageArray &ImageInput2, const HImageArray &ImageInput3, HImageArray *ImageGreen, HImageArray *ImageBlue, const char *ColorSpace) const;
  virtual HImageArray PowerLn(void) const;
  virtual HImageArray PowerReal(void) const;
  virtual HImageArray PowerByte(void) const;
  virtual HImageArray PhaseDeg(void) const;
  virtual HImageArray PhaseRad(void) const;
  virtual HImageArray EnergyGabor(const HImageArray &ImageHilbert) const;
  virtual HImageArray ConvolGabor(const HImage &GaborFilter, HImageArray *ImageResultHilbert) const;
  virtual HImageArray CorrelationFft(const HImageArray &ImageFFT2) const;
  virtual HImageArray ConvolFft(const HImage &ImageFilter) const;
  virtual HImageArray RftGeneric(const HTuple &Direction, const HTuple &Norm, const HTuple &ResultType, const HTuple &Width) const;
  virtual HImageArray RftGeneric(const char *Direction, const char *Norm, const char *ResultType, long Width) const;
  virtual HImageArray FftImageInv(void) const;
  virtual HImageArray FftImage(void) const;
  virtual HImageArray FftGeneric(const HTuple &Direction, const HTuple &Exponent, const HTuple &Norm, const HTuple &Mode, const HTuple &ResultType) const;
  virtual HImageArray FftGeneric(const char *Direction, long Exponent, const char *Norm, const char *Mode, const char *ResultType) const;
  virtual HImageArray SmoothImage(const HTuple &Filter, const HTuple &Alpha) const;
  virtual HImageArray SmoothImage(const char *Filter, double Alpha) const;
  virtual HImageArray SigmaImage(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &Sigma) const;
  virtual HImageArray SigmaImage(long MaskHeight, long MaskWidth, long Sigma) const;
  virtual HImageArray MidrangeImage(const HRegion &Mask, const HTuple &Margin) const;
  virtual HImageArray TrimmedMean(const HRegion &Mask, const HTuple &Number, const HTuple &Margin) const;
  virtual HImageArray MedianSeparate(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &Margin) const;
  virtual HImageArray MedianImage(const HTuple &MaskType, const HTuple &Radius, const HTuple &Margin) const;
  virtual HImageArray MedianWeighted(const HTuple &MaskType, const HTuple &MaskSize) const;
  virtual HImageArray MedianWeighted(const char *MaskType, long MaskSize) const;
  virtual HImageArray RankImage(const HRegion &Mask, const HTuple &Rank, const HTuple &Margin) const;
  virtual HImageArray DualRank(const HTuple &MaskType, const HTuple &Radius, const HTuple &ModePercent, const HTuple &Margin) const;
  virtual HImageArray MeanImage(const HTuple &MaskWidth, const HTuple &MaskHeight) const;
  virtual HImageArray MeanImage(long MaskWidth, long MaskHeight) const;
  virtual HImageArray GaussImage(const HTuple &Size) const;
  virtual HImageArray GaussImage(long Size) const;
  virtual HImageArray AnisotropeDiff(const HTuple &Percent, const HTuple &Mode, const HTuple &Iteration, const HTuple &NeighborhoodType) const;
  virtual HImageArray AnisotropeDiff(long Percent, long Mode, long Iteration, long NeighborhoodType) const;
  virtual HImageArray FillInterlace(const HTuple &Mode) const;
  virtual HImageArray FillInterlace(const char *Mode) const;
  virtual HImageArray MeanN(void) const;
  virtual HImageArray EliminateSp(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &MinThresh, const HTuple &MaxThresh) const;
  virtual HImageArray EliminateSp(long MaskWidth, long MaskHeight, long MinThresh, long MaxThresh) const;
  virtual HImageArray MeanSp(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &MinThresh, const HTuple &MaxThresh) const;
  virtual HImageArray MeanSp(long MaskWidth, long MaskHeight, long MinThresh, long MaxThresh) const;
  virtual HImageArray SobelAmp(const HTuple &FilterType, const HTuple &Size) const;
  virtual HImageArray SobelAmp(const char *FilterType, long Size) const;
  virtual HImageArray SobelDir(HImageArray *EdgeDirection, const HTuple &FilterType, const HTuple &Size) const;
  virtual HImageArray SobelDir(HImageArray *EdgeDirection, const char *FilterType, long Size) const;
  virtual HImageArray Roberts(const HTuple &FilterType) const;
  virtual HImageArray Roberts(const char *FilterType) const;
  virtual HImageArray Laplace(const HTuple &FilterType, const HTuple &Size, const HTuple &NeighbourhoodType) const;
  virtual HImageArray Laplace(const char *FilterType, long Size, const char *NeighbourhoodType) const;
  virtual HImageArray HighpassImage(const HTuple &Width, const HTuple &Height) const;
  virtual HImageArray HighpassImage(long Width, long Height) const;
  virtual HImageArray EdgesColor(HImageArray *ImaDir, const HTuple &Filter, const HTuple &Alpha, const HTuple &NMS, const HTuple &Low, const HTuple &High) const;
  virtual HImageArray EdgesColor(HImageArray *ImaDir, const char *Filter, double Alpha, const char *NMS, long Low, long High) const;
  virtual HImageArray EdgesImage(HImageArray *ImaDir, const HTuple &Filter, const HTuple &Alpha, const HTuple &NMS, const HTuple &Low, const HTuple &High) const;
  virtual HImageArray EdgesImage(HImageArray *ImaDir, const char *Filter, double Alpha, const char *NMS, long Low, long High) const;
  virtual HImageArray DerivateGauss(const HTuple &Sigma, const HTuple &Component) const;
  virtual HImageArray DerivateGauss(double Sigma, const char *Component) const;
  virtual HImageArray LaplaceOfGauss(const HTuple &Sigma) const;
  virtual HImageArray LaplaceOfGauss(double Sigma) const;
  virtual HImageArray DiffOfGauss(const HTuple &Sigma, const HTuple &SigFactor) const;
  virtual HImageArray DiffOfGauss(double Sigma, double SigFactor) const;
  virtual HTuple DetectEdgeSegments(const HTuple &SobelSize, const HTuple &MinAmplitude, const HTuple &MaxDistance, const HTuple &MinLength, HTuple *BeginCol, HTuple *EndRow, HTuple *EndCol) const;
  virtual HTuple DetectEdgeSegments(long SobelSize, long MinAmplitude, long MaxDistance, long MinLength, HTuple *BeginCol, HTuple *EndRow, HTuple *EndCol) const;
  virtual HImageArray RobinsonDir(HImageArray *ImageEdgeDir) const;
  virtual HImageArray RobinsonAmp(void) const;
  virtual HImageArray KirschDir(HImageArray *ImageEdgeDir) const;
  virtual HImageArray KirschAmp(void) const;
  virtual HImageArray FreiDir(HImageArray *ImageEdgeDir) const;
  virtual HImageArray FreiAmp(void) const;
  virtual HImageArray PrewittDir(HImageArray *ImageEdgeDir) const;
  virtual HImageArray PrewittAmp(void) const;
  virtual HImageArray BandpassImage(const HTuple &FilterType) const;
  virtual HImageArray BandpassImage(const char *FilterType) const;
  virtual HImageArray OpticalFlowMatch(const HImageArray &Image2, const HTuple &Mode, const HTuple &Threshold, const HTuple &Step, const HTuple &SizeWindow, const HTuple &SizeSearch, const HTuple &Weights) const;
  virtual HImageArray OpticalFlowMatch(const HImageArray &Image2, const char *Mode, long Threshold, long Step, long SizeWindow, long SizeSearch, long Weights) const;
  virtual HImageArray ExhaustiveMatchMg(const HImage &ImageTemplate, const HTuple &Mode, const HTuple &Level, const HTuple &Threshold) const;
  virtual HImageArray ExhaustiveMatchMg(const HImage &ImageTemplate, const char *Mode, long Level, long Threshold) const;
  virtual void AdaptTemplate(const HTuple &TemplateID) const;
  virtual void AdaptTemplate(long TemplateID) const;
  virtual HRegionArray FastMatchMg(const HTuple &TemplateID, const HTuple &MaxError, const HTuple &NumLevel) const;
  virtual HRegionArray FastMatchMg(long TemplateID, double MaxError, long NumLevel) const;
  // Searching the best grayvalue matches in a pre generated pyramid.
  virtual double BestMatchPreMg(const HTuple &TemplateID, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &WhichLevels, HTuple *Column, HTuple *Error) const;
  virtual double BestMatchPreMg(long TemplateID, double MaxError, const char *SubPixel, long NumLevels, long WhichLevels, HTuple *Column, HTuple *Error) const;
  virtual double BestMatchMg(const HTuple &TemplateID, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &WhichLevels, HTuple *Column, HTuple *Error) const;
  virtual double BestMatchMg(long TemplateID, double MaxError, const char *SubPixel, long NumLevels, long WhichLevels, HTuple *Column, HTuple *Error) const;
  virtual HRegionArray FastMatch(const HTuple &TemplateID, const HTuple &MaxError) const;
  virtual HRegionArray FastMatch(long TemplateID, double MaxError) const;
  virtual HTuple BestMatchRotMg(const HTuple &TemplateID, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &MaxError, const HTuple &SubPixel, const HTuple &NumLevels, HTuple *Column, HTuple *Angle, HTuple *Error) const;
  virtual HTuple BestMatchRotMg(long TemplateID, double AngleStart, double AngleExtend, double MaxError, const char *SubPixel, long NumLevels, HTuple *Column, HTuple *Angle, HTuple *Error) const;
  virtual HTuple BestMatchRot(const HTuple &TemplateID, const HTuple &AngleStart, const HTuple &AngleExtend, const HTuple &MaxError, const HTuple &SubPixel, HTuple *Column, HTuple *Angle, HTuple *Error) const;
  virtual HTuple BestMatchRot(long TemplateID, double AngleStart, double AngleExtend, double MaxError, const char *SubPixel, HTuple *Column, HTuple *Angle, HTuple *Error) const;
  virtual HTuple BestMatch(const HTuple &TemplateID, const HTuple &MaxError, const HTuple &SubPixel, HTuple *Column, HTuple *Error) const;
  virtual HTuple BestMatch(long TemplateID, double MaxError, const char *SubPixel, HTuple *Column, HTuple *Error) const;
  virtual HImageArray CornerResponse(const HTuple &Size, const HTuple &Weight) const;
  virtual HImageArray CornerResponse(long Size, double Weight) const;
  virtual HImageArray Monotony(void) const;
  virtual HImageArray FillDvf(const HTuple &Width, const HTuple &Height, const HTuple &Iterations) const;
  virtual HImageArray FillDvf(long Width, long Height, long Iterations) const;
  virtual HImageArray AddNoiseWhite(const HTuple &Amp) const;
  virtual HImageArray AddNoiseWhite(double Amp) const;
  virtual HImageArray AddNoiseDistribution(const HTuple &Distribution) const;
  virtual HImageArray AddNoiseDistribution(double Distribution) const;
  virtual HImageArray DotsImage(const HTuple &Diameter, const HTuple &FilterType, const HTuple &PixelShift) const;
  virtual HImageArray DotsImage(long Diameter, const char *FilterType, long PixelShift) const;
  virtual HImageArray ExpandDomainGray(const HTuple &ExpansionRange) const;
  virtual HImageArray ExpandDomainGray(long ExpansionRange) const;
  virtual HImageArray TopographicSketch(void) const;
  virtual HImageArray GrayClosingShape(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &MaskShape) const;
  virtual HImageArray GrayClosingShape(long MaskHeight, long MaskWidth, const char *MaskShape) const;
  virtual HImageArray GrayOpeningShape(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &MaskShape) const;
  virtual HImageArray GrayOpeningShape(long MaskHeight, long MaskWidth, const char *MaskShape) const;
  virtual HImageArray GrayErosionShape(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &MaskShape) const;
  virtual HImageArray GrayErosionShape(long MaskHeight, long MaskWidth, const char *MaskShape) const;
  virtual HImageArray GrayDilationShape(const HTuple &MaskHeight, const HTuple &MaskWidth, const HTuple &MaskShape) const;
  virtual HImageArray GrayDilationShape(long MaskHeight, long MaskWidth, const char *MaskShape) const;
  virtual HImageArray GrayRangeRect(const HTuple &MaskHeight, const HTuple &MaskWidth) const;
  virtual HImageArray GrayRangeRect(long MaskHeight, long MaskWidth) const;
  virtual HImageArray GrayErosionRect(const HTuple &MaskHeight, const HTuple &MaskWidth) const;
  virtual HImageArray GrayErosionRect(long MaskHeight, long MaskWidth) const;
  virtual HImageArray GrayDilationRect(const HTuple &MaskHeight, const HTuple &MaskWidth) const;
  virtual HImageArray GrayDilationRect(long MaskHeight, long MaskWidth) const;
  virtual HImageArray GraySkeleton(void) const;
  virtual HImageArray LutTrans(const HTuple &Lut) const;
  virtual HImageArray LutTrans(long Lut) const;
  virtual HImageArray ConvolImage(const HTuple &FilterMask, const HTuple &Margin) const;
  virtual HImageArray ConvertImageType(const HTuple &NewType) const;
  virtual HImageArray ConvertImageType(const char *NewType) const;
  virtual HImageArray Int1ToDvf(const HImageArray &Col) const;
  virtual HImageArray DvfToInt1(HImageArray *Col) const;
  virtual HImageArray RealToComplex(const HImageArray &ImageImaginary) const;
  virtual HImageArray ComplexToReal(HImageArray *ImageImaginary) const;
  virtual HImageArray GrayInside(void) const;
  virtual HImageArray Symmetry(const HTuple &MaskSize, const HTuple &Direction, const HTuple &Exponent) const;
  virtual HImageArray Symmetry(long MaskSize, double Direction, double Exponent) const;
  virtual HImageArray SelectGrayvaluesFromChannels(const HImageArray &IndexImage) const;
  virtual HImageArray DepthFromFocus(HImageArray *Confidence, const HTuple &Filter, const HTuple &Selection) const;
  virtual HImageArray DepthFromFocus(HImageArray *Confidence, const char *Filter, const char *Selection) const;
  virtual HImageArray TextureLaws(const HTuple &FilterTypes, const HTuple &Shift, const HTuple &FilterSize) const;
  virtual HImageArray TextureLaws(const char *FilterTypes, long Shift, long FilterSize) const;
  virtual HImageArray DeviationImage(const HTuple &Width, const HTuple &Height) const;
  virtual HImageArray DeviationImage(long Width, long Height) const;
  virtual HImageArray EntropyImage(const HTuple &Width, const HTuple &Height) const;
  virtual HImageArray EntropyImage(long Width, long Height) const;
  virtual HImageArray EquHistoImage(void) const;
  virtual HImageArray Illuminate(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &Factor) const;
  virtual HImageArray Illuminate(long MaskWidth, long MaskHeight, double Factor) const;
  virtual HImageArray Emphasize(const HTuple &MaskWidth, const HTuple &MaskHeight, const HTuple &Factor) const;
  virtual HImageArray Emphasize(long MaskWidth, long MaskHeight, double Factor) const;
  virtual HImageArray ScaleImageMax(void) const;
  virtual void DispChannel(const HTuple &WindowHandle, const HTuple &Channel) const;
  virtual void DispChannel(long WindowHandle, long Channel) const;
  virtual HImageArray MapImage(const HImage &Map) const;
  virtual HImageArray ImageToWorldPlane(const HTuple &CamParam, const HTuple &CamPose, const HTuple &Width, const HTuple &Height, const HTuple &Scale, const HTuple &Interpolation) const;
  virtual HImageArray ChangeRadialDistortionImage(const HRegion &Region, const HTuple &CamParIn, const HTuple &CamParOut) const;
  virtual HImageArray ChangeRadialDistortionImage(const HRegion &Region, double CamParIn, double CamParOut) const;
  virtual HRegionArray ClusterModelComponents(const HTuple &ComponentTrainingID, const HTuple &AmbiguityCriterion, const HTuple &MaxContourOverlap, const HTuple &ClusterThreshold) const;
  virtual HRegionArray ClusterModelComponents(long ComponentTrainingID, const char *AmbiguityCriterion, double MaxContourOverlap, double ClusterThreshold) const;
  virtual HRegionArray GenInitialComponents(const HTuple &ContrastLow, const HTuple &ContrastHigh, const HTuple &MinSize, const HTuple &Mode, const HTuple &GenericName, const HTuple &GenericValue) const;
  virtual HRegionArray GenInitialComponents(long ContrastLow, long ContrastHigh, long MinSize, const char *Mode, const char *GenericName, double GenericValue) const;
  virtual HImageArray GrayBothat(const HImage &SE) const;
  virtual HImageArray GrayTophat(const HImage &SE) const;
  virtual HImageArray GrayClosing(const HImage &SE) const;
  virtual HImageArray GrayOpening(const HImage &SE) const;
  virtual HImageArray GrayDilation(const HImage &SE) const;
  virtual HImageArray GrayErosion(const HImage &SE) const;
  virtual HImageArray CropDomainRel(const HTuple &Top, const HTuple &Left, const HTuple &Bottom, const HTuple &Right) const;
  virtual HImageArray CropDomainRel(long Top, long Left, long Bottom, long Right) const;
  virtual void WriteOcrTrainfImage(const HTuple &Class, const HTuple &FileName) const;
  virtual void WriteOcrTrainfImage(const char *Class, const char *FileName) const;
  // Read training specific characters from files and convert to images.
  static HImageArray ReadOcrTrainfSelect(const HTuple &TrainFileNames, const HTuple &SearchNames, HTuple *FoundNames);
  static HImageArray ReadOcrTrainfSelect(const char *TrainFileNames, const char *SearchNames, HTuple *FoundNames);
  // Read training characters from files and convert to images.
  static HImageArray ReadOcrTrainf(const HTuple &TrainFileNames, HTuple *CharacterNames);
  static HImageArray ReadOcrTrainf(const char *TrainFileNames, HTuple *CharacterNames);
  virtual HTuple DoOcvSimple(const HTuple &OCVHandle, const HTuple &PatternName, const HTuple &AdaptPos, const HTuple &AdaptSize, const HTuple &AdaptAngle, const HTuple &AdaptGray, const HTuple &Threshold) const;
  virtual HTuple DoOcvSimple(long OCVHandle, const char *PatternName, const char *AdaptPos, const char *AdaptSize, const char *AdaptAngle, const char *AdaptGray, double Threshold) const;
  virtual void TraindOcvProj(const HTuple &OCVHandle, const HTuple &Name, const HTuple &Mode) const;
  virtual void TraindOcvProj(long OCVHandle, const char *Name, const char *Mode) const;
  virtual HRegionArray ClassNdimBox(const HTuple &ClassifHandle) const;
  virtual HRegionArray ClassNdimBox(long ClassifHandle) const;
  virtual HRegionArray ClassNdimNorm(const HTuple &Metric, const HTuple &SingleMultiple, const HTuple &Radius, const HTuple &Center) const;
  virtual HRegionArray ClassNdimNorm(const char *Metric, const char *SingleMultiple, double Radius, double Center) const;
  virtual HRegionArray Class2dimSup(const HImageArray &ImageRow, const HRegionArray &FeatureSpace) const;
  virtual HRegionArray CheckDifference(const HImageArray &Pattern, const HTuple &Mode, const HTuple &DiffLowerBound, const HTuple &DiffUpperBound, const HTuple &GrayOffset, const HTuple &AddRow, const HTuple &AddCol) const;
  virtual HRegionArray CheckDifference(const HImageArray &Pattern, const char *Mode, long DiffLowerBound, long DiffUpperBound, long GrayOffset, long AddRow, long AddCol) const;
  virtual HRegionArray CharThreshold(const HRegion &HistoRegion, const HTuple &Sigma, const HTuple &Percent, HTuple *Threshold) const;
  virtual HRegionArray CharThreshold(const HRegion &HistoRegion, double Sigma, double Percent, HTuple *Threshold) const;
  virtual HRegionArray LabelToRegion(void) const;
  virtual HImageArray NonmaxSuppressionAmp(const HTuple &Mode) const;
  virtual HImageArray NonmaxSuppressionAmp(const char *Mode) const;
  virtual HImageArray NonmaxSuppressionDir(const HImageArray &ImgDir, const HTuple &Mode) const;
  virtual HImageArray NonmaxSuppressionDir(const HImageArray &ImgDir, const char *Mode) const;
  virtual HRegionArray HysteresisThreshold(const HTuple &Low, const HTuple &High, const HTuple &MaxLength) const;
  virtual HRegionArray HysteresisThreshold(long Low, long High, long MaxLength) const;
  virtual HRegionArray DynThreshold(const HImageArray &ThresholdImage, const HTuple &Offset, const HTuple &LightDark) const;
  virtual HRegionArray DynThreshold(const HImageArray &ThresholdImage, double Offset, const char *LightDark) const;
  virtual HRegionArray Threshold(const HTuple &MinGray, const HTuple &MaxGray) const;
  virtual HRegionArray Threshold(double MinGray, double MaxGray) const;
  virtual HRegionArray RegiongrowingN(const HTuple &Metric, const HTuple &MinTolerance, const HTuple &MaxTolerance, const HTuple &MinSize) const;
  virtual HRegionArray RegiongrowingN(const char *Metric, double MinTolerance, double MaxTolerance, long MinSize) const;
  virtual HRegionArray Regiongrowing(const HTuple &Row, const HTuple &Column, const HTuple &Tolerance, const HTuple &MinSize) const;
  virtual HRegionArray Regiongrowing(long Row, long Column, double Tolerance, long MinSize) const;
  virtual HRegionArray RegiongrowingMean(const HTuple &StartRows, const HTuple &StartColumns, const HTuple &Tolerance, const HTuple &MinSize) const;
  virtual HRegionArray RegiongrowingMean(long StartRows, long StartColumns, double Tolerance, long MinSize) const;
  virtual HRegionArray Watersheds(HRegionArray *Watersheds) const;
  virtual HRegionArray ZeroCrossing(void) const;
  virtual HRegionArray DualThreshold(const HTuple &MinSize, const HTuple &MinGray, const HTuple &Threshold) const;
  virtual HRegionArray DualThreshold(long MinSize, double MinGray, double Threshold) const;
  virtual HRegionArray ExpandLine(const HTuple &Index, const HTuple &ExpandType, const HTuple &RowColumn, const HTuple &Threshold) const;
  virtual HRegionArray ExpandLine(long Index, const char *ExpandType, const char *RowColumn, double Threshold) const;
  virtual HRegionArray LocalMin(void) const;
  virtual HRegionArray Lowlands(void) const;
  virtual HRegionArray LowlandsCenter(void) const;
  virtual HRegionArray LocalMax(void) const;
  virtual HRegionArray Plateaus(void) const;
  virtual HRegionArray PlateausCenter(void) const;
  virtual HRegionArray AutoThreshold(const HTuple &Sigma) const;
  virtual HRegionArray AutoThreshold(double Sigma) const;
  virtual HRegionArray BinThreshold(void) const;
  virtual HRegionArray FastThreshold(const HTuple &MinGray, const HTuple &MaxGray, const HTuple &MinHeight) const;
  virtual HRegionArray FastThreshold(double MinGray, double MaxGray, long MinHeight) const;
  virtual HImageArray ShadeHeightField(const HTuple &Slant, const HTuple &Tilt, const HTuple &Albedo, const HTuple &Ambient, const HTuple &Shadows) const;
  virtual HImageArray ShadeHeightField(double Slant, double Tilt, double Albedo, double Ambient, const char *Shadows) const;
  virtual HTuple EstimateAlAm(HTuple *Ambient) const;
  virtual HTuple EstimateSlAlZc(HTuple *Albedo) const;
  virtual HTuple EstimateSlAlLr(HTuple *Albedo) const;
  virtual HTuple EstimateTiltZc(void) const;
  virtual HTuple EstimateTiltLr(void) const;
  virtual HImageArray SfsPentland(const HTuple &Slant, const HTuple &Tilt, const HTuple &Albedo, const HTuple &Ambient) const;
  virtual HImageArray SfsPentland(double Slant, double Tilt, double Albedo, double Ambient) const;
  virtual HImageArray SfsOrigLr(const HTuple &Slant, const HTuple &Tilt, const HTuple &Albedo, const HTuple &Ambient) const;
  virtual HImageArray SfsOrigLr(double Slant, double Tilt, double Albedo, double Ambient) const;
  virtual HImageArray SfsModLr(const HTuple &Slant, const HTuple &Tilt, const HTuple &Albedo, const HTuple &Ambient) const;
  virtual HImageArray SfsModLr(double Slant, double Tilt, double Albedo, double Ambient) const;
  static HImageArray ReceiveImage(const HTuple &Socket);
  static HImageArray ReceiveImage(long Socket);
  virtual void SendImage(const HTuple &Socket) const;
  virtual void SendImage(long Socket) const;
  virtual HTuple FindScaledShapeModels(const HTuple &ModelIDs, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &ScaleMin, const HTuple &ScaleMax, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score, HTuple *Model) const;
  virtual HTuple FindScaledShapeModels(long ModelIDs, double AngleStart, double AngleExtent, double ScaleMin, double ScaleMax, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score, HTuple *Model) const;
  virtual HTuple FindShapeModels(const HTuple &ModelIDs, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Score, HTuple *Model) const;
  virtual HTuple FindShapeModels(long ModelIDs, double AngleStart, double AngleExtent, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Score, HTuple *Model) const;
};

#endif
