/*****************************************************************************
 * HOCR.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HOCR
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 09:57:10 $
 *
 */

#ifndef H_OCR_H
#define H_OCR_H

class LIntExport HOCR: public HRootObject {
public:
  HOCR(void) {id = -1;}
  HOCR(const HTuple &WidthPattern, const HTuple &HeightPattern, 
       const HTuple &Interpolation, 
       const HTuple &Features, 
       const HTuple &Character);

  ~HOCR(void);
private:
  // Copying and Assignment not allowed
  HOCR(const HOCR &inst);
  HOCR &operator= (const HOCR &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HOCR";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Writing an OCR classifier into a file.
  virtual void WriteOcr(const HTuple &FileName) const;
  virtual void WriteOcr(const char *FileName) const;
  // Reading of an OCR classifier from a file.
  virtual void ReadOcr(const HTuple &FileName);
  virtual void ReadOcr(const char *FileName);
  // Classification of a character.
  virtual HTuple DoOcrSingle(const HRegion &Character, const HImage &Image, HTuple *Confidences) const;
  // Classification of characters.
  virtual HTuple DoOcrMulti(const HRegionArray &Character, const HImage &Image, HTuple *Confidence) const;
  virtual HTuple DoOcrMulti(const HRegionArray &Character, const HImage &Image, double *Confidence) const;
  // Information about an OCR classifier.
  virtual long InfoOcrClassBox(HTuple *HeightPattern, HTuple *Interpolation, HTuple *WidthMaxChar, HTuple *HeightMaxChar, HTuple *Features, HTuple *Characters) const;
  virtual long InfoOcrClassBox(long *HeightPattern, long *Interpolation, long *WidthMaxChar, long *HeightMaxChar, HTuple *Features, HTuple *Characters) const;
  // Creating a new OCR-classifier.
  virtual void CreateOcrClassBox(const HTuple &WidthPattern, const HTuple &HeightPattern, const HTuple &Interpolation, const HTuple &Features, const HTuple &Character);
  // Training of an OCR classifier by the input of regions.
  virtual double TraindOcrClassBox(const HRegionArray &Character, const HImage &Image, const HTuple &Class) const;
  virtual double TraindOcrClassBox(const HRegionArray &Character, const HImage &Image, const char *Class) const;
  // Training of an OCR classifier with the help of a training file.
  virtual double TrainfOcrClassBox(const HTuple &FileName) const;
  virtual double TrainfOcrClassBox(const char *FileName) const;
  // Defining a new conversion table for the characters.
  virtual void OcrChangeChar(const HTuple &Character) const;
  // Training of an OCR classifier by the input of regions.
  virtual HTuple TestdOcrClassBox(const HRegionArray &Character, const HImage &Image, const HTuple &Class) const;
  virtual HTuple TestdOcrClassBox(const HRegionArray &Character, const HImage &Image, const char *Class) const;
  // Access the features which correspond to a character.
  virtual HTuple OcrGetFeatures(const HImage &Character) const;
};

#endif
