/*****************************************************************************
 * HOCRMLP.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HOCRMlp
 *
 * (c) 1996-2003 by MVTec Software GmbH
 *                  www.mvtec.com
 * 
 *****************************************************************************
 *
 * $Revision: 1.2 $
 * $Date: 2003/10/02 09:57:45 $
 *
 */

#ifndef H_OCRMLP_H
#define H_OCRMLP_H

class LIntExport HOCRMlp: public HRootObject {
public:
  HOCRMlp(void) {id = -1;}
  HOCRMlp(const HTuple &WidthCharacter, const HTuple &HeightCharacter, 
          const HTuple &Interpolation, const HTuple &Features, 
          const HTuple &Characters, const HTuple &NumHidden,
          const HTuple &Preprocessing, const HTuple &NumComponents,
          const HTuple &RandSeed);

  ~HOCRMlp(void);
private:
  // Copying and Assignment not allowed
  HOCRMlp(const HOCRMlp &inst);
  HOCRMlp &operator= (const HOCRMlp &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HOCRMlp";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Read an OCR classifier from a file.
  virtual void ReadOcrClassMlp(const HTuple &FileName);
  virtual void ReadOcrClassMlp(const char *FileName);
  // Write an OCR classifier to a file.
  virtual void WriteOcrClassMlp(const HTuple &FileName) const;
  virtual void WriteOcrClassMlp(const char *FileName) const;
  // Compute the features of a character.
  virtual HTuple GetFeaturesOcrClassMlp(const HImage &Character, const HTuple &Transform) const;
  virtual HTuple GetFeaturesOcrClassMlp(const HImage &Character, const char *Transform) const;
  // Classify multiple characters with an OCR classifier.
  virtual HTuple DoOcrMultiClassMlp(const HRegionArray &Character, const HImage &Image, HTuple *Confidence) const;
  virtual HTuple DoOcrMultiClassMlp(const HRegionArray &Character, const HImage &Image, double *Confidence) const;
  // Classify a single character with an OCR classifier.
  virtual HTuple DoOcrSingleClassMlp(const HRegion &Character, const HImage &Image, const HTuple &Num, HTuple *Confidence) const;
  virtual HTuple DoOcrSingleClassMlp(const HRegion &Character, const HImage &Image, const HTuple &Num, double *Confidence) const;
  // Train an OCR classifier.
  virtual double TrainfOcrClassMlp(const HTuple &TrainingFile, const HTuple &MaxIterations, const HTuple &WeightTolerance, const HTuple &ErrorTolerance, HTuple *ErrorLog) const;
  virtual double TrainfOcrClassMlp(const char *TrainingFile, long MaxIterations, double WeightTolerance, double ErrorTolerance, HTuple *ErrorLog) const;
  // Compute the information content of the preprocessed feature vectors
  // of an OCR classifier.
  virtual HTuple GetPrepInfoOcrClassMlp(const HTuple &TrainingFile, const HTuple &Preprocessing, HTuple *CumInformationCont) const;
  virtual HTuple GetPrepInfoOcrClassMlp(const char *TrainingFile, const char *Preprocessing, HTuple *CumInformationCont) const;
  // Return the parameters of an OCR classifier.
  virtual long GetParamsOcrClassMlp(HTuple *HeightCharacter, HTuple *Interpolation, HTuple *Features, HTuple *Characters, HTuple *NumHidden, HTuple *Preprocessing, HTuple *NumComponents) const;
  virtual long GetParamsOcrClassMlp(long *HeightCharacter, char *Interpolation, char *Features, HTuple *Characters, long *NumHidden, char *Preprocessing, long *NumComponents) const;
  // Create an OCR classifier using a multilayer perceptron.
  virtual void CreateOcrClassMlp(const HTuple &WidthCharacter, const HTuple &HeightCharacter, const HTuple &Interpolation, const HTuple &Features, const HTuple &Characters, const HTuple &NumHidden, const HTuple &Preprocessing, const HTuple &NumComponents, const HTuple &RandSeed);
};

#endif
