/*****************************************************************************
 * HSerial.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HSerial
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 09:58:43 $
 *
 */

#ifndef H_SERIAL_H
#define H_SERIAL_H

class LIntExport HSerial: public HRootObject {
public:
  HSerial(void) {id = -1;}
  HSerial(const HTuple &PortName);
  HSerial(const char *PortName);

  ~HSerial(void);
private:
  // Copying and Assignment not allowed
  HSerial(const HSerial &inst);
  HSerial &operator= (const HSerial &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HSerial";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Clear the buffer of a serial connection.
  virtual void ClearSerial(const HTuple &Channel) const;
  virtual void ClearSerial(const char *Channel) const;
  // Write to a serial connection.
  virtual void WriteSerial(const HTuple &Data) const;
  virtual void WriteSerial(long Data) const;
  // Read from a serial device.
  virtual HTuple ReadSerial(const HTuple &NumCharacters) const;
  virtual HTuple ReadSerial(long NumCharacters) const;
  // Get the parameters of a serial device.
  virtual long GetSerialParam(HTuple *DataBits, HTuple *FlowControl, HTuple *Parity, HTuple *StopBits, HTuple *TotalTimeOut, HTuple *InterCharTimeOut) const;
  virtual long GetSerialParam(long *DataBits, char *FlowControl, char *Parity, long *StopBits, long *TotalTimeOut, long *InterCharTimeOut) const;
  // Set the parameters of a serial device.
  virtual void SetSerialParam(const HTuple &BaudRate, const HTuple &DataBits, const HTuple &FlowControl, const HTuple &Parity, const HTuple &StopBits, const HTuple &TotalTimeOut, const HTuple &InterCharTimeOut) const;
  virtual void SetSerialParam(long BaudRate, long DataBits, const char *FlowControl, const char *Parity, long StopBits, long TotalTimeOut, long InterCharTimeOut) const;
  // Open a serial device.
  virtual void OpenSerial(const HTuple &PortName);
  virtual void OpenSerial(const char *PortName);
};

#endif
