/*****************************************************************************
 * HVariationModel.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HVariationModel
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 10:01:01 $
 *
 */

#ifndef H_VARIATION_MODEL_H
#define H_VARIATION_MODEL_H

class LIntExport HVariationModel: public HRootObject {
public:
  HVariationModel(void) {id = -1;}
  HVariationModel(const HTuple &Width, const HTuple &Height, 
                  const HTuple &Type = "byte", 
                  const HTuple &Mode = "standard");
  HVariationModel(long Width, long Height, 
                  const char *Type = "byte", 
                  const char *Mode = "standard");

  ~HVariationModel(void);
private:
  // Copying and Assignment not allowed
  HVariationModel(const HVariationModel &inst);
  HVariationModel &operator= (const HVariationModel &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HVariationModel";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  //Handle
  long  id;
public:
  // Read a variation model from a file.
  virtual void ReadVariationModel(const HTuple &FileName);
  virtual void ReadVariationModel(const char *FileName);
  // Write a variation model to a file.
  virtual void WriteVariationModel(const HTuple &FileName) const;
  virtual void WriteVariationModel(const char *FileName) const;
  // Return the images used for image comparison by a variation model.
  virtual HImage GetVariationModel(HImage *VarImage) const;
  // Compare an image to a variation model.
  virtual HRegionArray CompareVariationModel(const HImageArray &Image) const;
  // Prepare a variation model for comparison with an image.
  virtual void PrepareVariationModel(const HTuple &AbsThreshold, const HTuple &VarThreshold) const;
  virtual void PrepareVariationModel(double AbsThreshold, double VarThreshold) const;
  // Train a variation model.
  virtual void TrainVariationModel(const HImageArray &Images) const;
  // Create a variation model for image comparison.
  virtual void CreateVariationModel(const HTuple &Width, const HTuple &Height, const HTuple &Type, const HTuple &Mode);
  virtual void CreateVariationModel(long Width, long Height, const char *Type, const char *Mode);
};

#endif
