/*****************************************************************************
 * HXLD.h
 ***************************************************************************** 
 *
 * Project:     Halcon/libhalcon
 * Description: Class HXLD
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 */

#ifndef XLD_H
#define XLD_H


/****************************************************************************/
/*                               HXLD                                       */
/****************************************************************************/

class HXLDArray;
class LIntExport HXLD: public HObject {
  friend class HXLDArray;
public:
  HXLD(void): HObject() {num_xld++;}
  HXLD(Hkey key, HBool checktype = H_TRUE);
  HXLD(const Hobject &obj, HBool checktype = H_TRUE);
  HXLD(const HXLD &xld);
  HXLD(const HXLDArray &arr);
  ~HXLD(void) {num_xld--;}

  HXLD &operator= (const HXLD &xld);
  HXLD &operator= (const HXLDArray &arr);

  // Tools
  virtual const char *HClassName(void) const;
  virtual const char *ClassName(void) const;
  virtual const char *InstClassName(void) const;
  virtual HBool       Shared(void) const;
  virtual void        Display(const HWindow &w) const;

  // Class Functions
  static long NumInst(void) {return num_xld;}

private:
  static long num_xld;

public:
  // Choose all contours or polygons containing a given pixel.
  virtual HXLD SelectXldPoint(const HTuple &Row, const HTuple &Column) const;
  virtual HXLD SelectXldPoint(double Row, double Column) const;
  // Test whether one or more contours or polygons enclose the given point(s).
  virtual long TestXldPoint(const HTuple &Row, const HTuple &Column) const;
  virtual long TestXldPoint(double Row, double Column) const;
  // Orientation of a contour.
  virtual double OrientationXld(void) const;
  // Shape features derived from the ellipse parameters.
  virtual double EccentricityXld(double *Bulkiness, double *StructureFactor) const;
  // Shape factor for the compactness of a contour.
  virtual double CompactnessXld(void) const;
  // Maximum distance between two contour points.
  virtual double DiameterXld(double *Column1, double *Row2, double *Column2, double *Diameter) const;
  // Shape factor for the convexity of a contour.
  virtual double ConvexityXld(void) const;
  // Shape factor for the circularity (similarity to a circle) of a contour.
  virtual double CircularityXld(void) const;
  // Parameters of the equivalent ellipse.
  virtual double EllipticAxisXld(double *Rb, double *Phi) const;
  // Smallest enclosing rectangle with arbitrary orientation.
  virtual double SmallestRectangle2Xld(double *Column, double *Phi, double *Length1, double *Length2) const;
  // Enclosing rectangle parallel to the coordinate axes.
  virtual double SmallestRectangle1Xld(double *Column1, double *Row2, double *Column2) const;
  // Smallest enclosing circle of a contour.
  virtual double SmallestCircleXld(double *Column, double *Radius) const;
  // Transform the shape of a contour.
  virtual HXLD ShapeTransXld(const HTuple &Type) const;
  virtual HXLD ShapeTransXld(const char *Type) const;
  // Length of contours or polygons.
  virtual double LengthXld(void) const;
  // Arbitrary geometric moments of contours or polygons.
  virtual double MomentsAnyXld(const HTuple &Mode, const HTuple &PointOrder, const HTuple &Area, const HTuple &CenterRow, const HTuple &CenterCol, const HTuple &P, const HTuple &Q) const;
  virtual double MomentsAnyXld(const char *Mode, const char *PointOrder, double Area, double CenterRow, double CenterCol, long P, long Q) const;
  // Geometric moments M20, M02, and M11 of contours or polygons.
  virtual double MomentsXld(double *M20, double *M02) const;
  // Area and center of gravity (centroid) of contours and polygons.
  virtual double AreaCenterXld(double *Row, double *Column, char *PointOrder) const;
  // Paint an XLD object in an image.
  virtual HImage PaintXld(const HImage &Image, const HTuple &Grayval) const;
  virtual HImage PaintXld(const HImage &Image, double Grayval) const;
  // Receive an XLD object over a socket connection.
  static HXLD ReceiveXld(const HTuple &Socket);
  static HXLD ReceiveXld(long Socket);
  // Send an XLD object over a socket connection.
  virtual void SendXld(const HTuple &Socket) const;
  virtual void SendXld(long Socket) const;
  // Return an XLD parallel's data (as lines).
  virtual HTuple GetParallelsXld(HTuple *Col1, HTuple *Length1, HTuple *Phi1, HTuple *Row2, HTuple *Col2, HTuple *Length2, HTuple *Phi2) const;
};

#endif
