/*****************************************************************************
 * HXLDPoly.h
 ***************************************************************************** 
 *
 * Project:     Halcon/libhalcon
 * Description: Class HXLDPoly
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.14 $
 * $Date: 2001/12/06 10:04:06 $
 *
 */

#ifndef XLD_POLY_H
#define XLD_POLY_H


/****************************************************************************/
/*                               HXLDPoly                                   */
/****************************************************************************/

class HXLDPolyArray;
class LIntExport HXLDPoly: public HXLD {
  friend class HXLDPolyArray;
public:
  HXLDPoly(void): HXLD() {num_poly++;}
  HXLDPoly(Hkey key);
  HXLDPoly(const Hobject &obj);
  HXLDPoly(const HXLDPoly &poly);
  HXLDPoly(const HXLDPolyArray &arr);
  HXLDPoly(const HXLD &xld);
  ~HXLDPoly(void) {num_poly--;}

  HXLDPoly &operator= (const HXLDPoly &poly);
  HXLDPoly &operator= (const HXLDPolyArray &arr);
  HXLDPoly &operator= (const HXLD &xld);

  // Tools
  virtual const char *HClassName(void) const;
  virtual const char *ClassName(void) const;
  virtual const char *InstClassName(void) const;
  virtual       INT4  ClassDBID(void) const;
  HBool         Shared(void) const;

  // Class Functions
  static long NumInst(void) {return num_poly;}

private:
  static long num_poly;

public:
  // Create a region from an XLD polygon.
  virtual HRegion GenRegionPolygonXld(const HTuple &Mode) const;
  virtual HRegion GenRegionPolygonXld(const char *Mode) const;
  // Read XLD polygons from a file in ARC/INFO generate format.
  static HXLDPoly ReadPolygonXldArcInfo(const HTuple &FileName);
  static HXLDPoly ReadPolygonXldArcInfo(const char *FileName);
  // Write XLD polygons to a file in ARC/INFO generate format.
  virtual void WritePolygonXldArcInfo(const HTuple &FileName) const;
  virtual void WritePolygonXldArcInfo(const char *FileName) const;
  // Return an XLD polygon's data (as lines).
  virtual HTuple GetLinesXld(HTuple *BeginCol, HTuple *EndRow, HTuple *EndCol, HTuple *Length, HTuple *Phi) const;
  // Return an XLD polygon's data.
  virtual HTuple GetPolygonXld(HTuple *Col, HTuple *Length, HTuple *Phi) const;
  // Split XLD contours at dominant points.
  virtual HXLDCont SplitContoursXld(const HTuple &Mode, const HTuple &Weight, const HTuple &Smooth) const;
  virtual HXLDCont SplitContoursXld(const char *Mode, long Weight, long Smooth) const;
  // Apply an arbitrary affine transformation to XLD polygons.
  virtual HXLDPoly AffineTransPolygonXld(const HTuple &HomMat2D) const;
  virtual HXLDPoly AffineTransPolygonXld(double HomMat2D) const;
};

#endif
